//Demo application to Read and Write RTC & EEPROM using Toradex  library for PXAxxx and Txx
#include <windows.h> 
#include "gpioLib.h"
#include "I2cLib.h"
typedef enum
{
	RTC_SELECTION ='a', 
	EEPROM_SELECTION = 'b',
	EXIT_SELECTION = 'c',
}SELECTION;

//******************************************************************************
/// RTC demo for testing i2c \n
BOOL rtcDemoOptionSelected()
{
	char *rtcDay[8] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
	BYTE rtcData[8] = {0};
	BYTE rtcAddress = 0x68;													  ///< Address of RTC
	BYTE sixBitMask = 0x3F;
	BYTE fiveBitMask = 0x1F;
	BYTE threeBitMask = 0x07;
	BYTE sevenBitMask = 0x7F;
	BOOL i2cError = FALSE;
	SYSTEMTIME dateTime;
	i2cError = I2CBurstRead(rtcAddress, rtcData, 0, 8);						  ///< Read RTC Chip on I2C 
	if (i2cError == FALSE)
	{
		printf("RTC read Failed\n");
		return FALSE;
	} 

	dateTime.wYear = (WORD)rtcData[6];
	dateTime.wMonth = (WORD)(rtcData[5] & fiveBitMask);
	dateTime.wDay = (WORD)(rtcData[4] & sixBitMask);
	dateTime.wHour = (WORD)(rtcData[2] & sixBitMask);
	dateTime.wMinute = (WORD)(rtcData[1] & sevenBitMask);
	dateTime.wSecond = (WORD)(rtcData[0] & sevenBitMask);
	dateTime.wDayOfWeek = (WORD)((rtcData[3] & threeBitMask) - 1);

	printf("Day is %s\n", rtcDay[dateTime.wDayOfWeek]);   
	printf("Today's date is %x/0%x/%x (DD/MM/YY)\n", dateTime.wDay, dateTime.wMonth, dateTime.wYear);
	printf("Time is %x:%x:%x (HH:MM:SS) \n", dateTime.wHour, dateTime.wMinute, dateTime.wSecond);
	return TRUE;
}
//******************************************************************************
/// External EEPROM demo for testing i2c \n
BOOL eepromDemoOptionSelected()
{
	BYTE eepromAddress = 0x50;													///< Device address for EEPROM
	BYTE eepromData[8] = {0x00, 0x00, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16};		///< First two bytes denote the EEPROM address where data will be written, next 6 bytes are the data to be written
	BYTE eepromDataRead[8] = {0, 0, 0, 0, 0, 0, 0, 0};							///< Buffer for data read
	BOOL i2cError = FALSE;
	i2cError = I2CBurstWrite(eepromAddress, eepromData, -1, 8);					///< Writes the 6 bytes of data to the device
	if (i2cError == FALSE)
	{
		printf("cannot perform I2C write operation\n");
		return FALSE;
	}
	Sleep(100);	
	printf("Data being written is %x %x %x %x %x %x\n", eepromData[2], eepromData[3], eepromData[4], eepromData[5], eepromData[6], eepromData[7]);
	i2cError = I2CBurstWrite(eepromAddress, eepromData, -1, 2);					///< Writes the first two bytes of eepromData i.e. the address to the EEPROM
	Sleep(10);																	///< Delay to finish write operation to EEPROM
	i2cError = I2CBurstRead(eepromAddress, eepromDataRead, -1, 6);				///< To read the 6 bytes of data written at the given address (0x00)
	if (i2cError == FALSE)
	{
		printf("cannot perform I2C read operation\n");
		return FALSE;
	}
	printf("Data read is %x %x %x %x %x %x\n", eepromDataRead[0], eepromDataRead[1], eepromDataRead[2], eepromDataRead[3], eepromDataRead[4], eepromDataRead[5]);
	return TRUE;
}
int wmain()
{
	BOOL i2cError = FALSE;
	DWORD verMaj = 0;
	DWORD verMin = 0;
	DWORD verBuild = 0;
	CHAR choice = ' ';

	I2CInit();																	///< Initialise the I2C bus 
	I2CSetSpeed(I2C_100KBPS);													///< Set I2C bus speed 
	I2CGetLibVersion(&verMaj, &verMin, &verBuild);
	printf("I2CDemo (I2CLib Version %d.%d.%d)\n", verMaj, verMin, verBuild);

	i2cError = GetI2CLock(5000);												///< Wait for 5000 ms to acquire I2C lock
	if (i2cError == FALSE)
	{
		printf("cannot acquire I2C lock\n");
		return FALSE;
	}
	while(choice != EXIT_SELECTION)
	{
		fflush(stdin);
		printf("\nEnter option\n");
		printf("\na.) RTC \nb.) EEPROM(External EEPROM)\nc.) EXIT\n");
		scanf_s("%c", &choice);
		switch(choice)
		{
		case RTC_SELECTION:
			rtcDemoOptionSelected();											///< functon for RTC Demo 
			break;
		case EEPROM_SELECTION:
			eepromDemoOptionSelected();											///< function for EEPROM Demo
			break;
		case EXIT_SELECTION:
			printf("\nPress Enter to close the program\n");
			getchar();
			break;
		default:
			printf("Invalid option\n");
			break;
		}
	}
	ReleaseI2CLock();															///< Release the I2C bus for other threads 
	I2CDeInit();
	return TRUE;
}